<?php

namespace jrosset\CliProgram\Validation\Validators;

use jrosset\Collections\Collection;
use jrosset\Collections\ICollection;

/**
 * An argument/option value validator based on a list of value
 *
 * @template TValue
 * @template-implements IValidator<TValue>
 * @template-implements TInternalValueValidator<TValue>
 */
class ListValidator implements IValidator {
    use TIdenticalValidDefaultValidator;
    use TInternalValueValidator;

    /**
     * @var ICollection The list of allowed values
     */
    private ICollection $allowedValues;

    /**
     * Create a validator
     *
     * @param ICollection|null $allowedValues The list of allowed values
     */
    public function __construct (?ICollection $allowedValues = null) {
        $this->setAllowedValues($allowedValues ?? new Collection());
    }

    /**
     * @inheritDoc
     */
    public function validate (mixed $value): bool {
        if ($this->getAllowedValues()->contains($value)) {
            $this->setValue($value);
            return true;
        }
        return false;
    }

    /**
     * The list of allowed values
     *
     * @return ICollection The list of allowed values
     */
    public function getAllowedValues (): ICollection {
        return $this->allowedValues;
    }
    /**
     * Set the list of allowed values
     *
     * @param ICollection $allowedValues The list of allowed values
     *
     * @return $this
     */
    public function setAllowedValues (ICollection $allowedValues): self {
        $this->allowedValues = $allowedValues;
        return $this;
    }
}
